# -*- coding: utf-8 -*-

import os, sys

BASE_PATH = os.path.abspath(os.path.dirname(__file__))
PKG_NAME = os.path.basename(BASE_PATH)


def configuration(parent_package='', top_path=None):
    from numpy.distutils.misc_util import Configuration

    config = Configuration(PKG_NAME, parent_package, top_path)

    # Add subpackage
    config.add_subpackage('cnt')
    config.add_subpackage('polygon')

    # Add test directories
    from os.path import isdir, dirname, join
    rel_isdir = lambda d: isdir(join(curpath, d))

    curpath = join(dirname(__file__), './')
    subdirs = [join(d, 'tests') for d in os.listdir(curpath) if rel_isdir(d)]
    subdirs = [d for d in subdirs if rel_isdir(d)]

    for test_dir in subdirs:
        config.add_data_dir(test_dir)

    return config


if __name__ == "__main__":
    from numpy.distutils.core import setup

    config = configuration(top_path='').todict()
    setup(**config)



# -*- coding: utf-8 -*-

import os, sys

from pycontour import point_list_to_np_arr
from pycontour import np_arr_to_cv_cnt

from pycontour.cnt import get_cnt_area
from pycontour.cnt import get_cnt_aspect_ratio
from pycontour.cnt import get_cnt_solidity

def test_property(cnt):
    cnt_area = get_cnt_area(cnt)
    cnt_aspect_ratio = get_cnt_aspect_ratio(cnt)
    cnt_solidty = get_cnt_solidity(cnt)
    print("Contour area is {}".format(cnt_area))
    print("Contour aspect ratio is {}".format(cnt_aspect_ratio))
    print("Contour solidity is {}".format(cnt_solidty))

if __name__ == "__main__":
    point_list1 = [(0, 1), (1, 2), (2, 1), (1, 0)]
    np_arr1 = point_list_to_np_arr(point_list1)
    cnt1 = np_arr_to_cv_cnt(np_arr1)
    test_property(cnt1)

    point_list2 = [(0, 0), (1, 2), (2, 0), (1, 0), (1, -1)]
    np_arr2 = point_list_to_np_arr(point_list2)
    cnt2 = np_arr_to_cv_cnt(np_arr2)
    test_property(cnt2)



# -*- coding: utf-8 -*-

import os, sys
from shapely.geometry import Polygon
from pycontour import construct_poly_using_np_arr

__all__ = ["contour_intersects", "construct_intersection_polygon",
        "contour_contains"]


def contour_intersects(np_arr1, np_arr2):
    """Determine two contours are intersected or not
    Parameters
    ----------    
    np_arr1 : np.array
        Contour with numpy array format
    np_arr2 : np.array
        Contour with numpy array format
    
    Returns
    -------
    inter_flag : bool
        Check if two contours are intersected or not.

    """
    # Construct polygon from numpy array
    poly1 = construct_poly_using_np_arr(np_arr1)
    poly2 = construct_poly_using_np_arr(np_arr2)
    # Get convex_hull of polygon
    poly1 = poly1.convex_hull  # Get convex hull of poly
    poly2 = poly2.convex_hull

    inter_flag = poly1.intersects(poly2)

    return inter_flag


def construct_intersection_polygon(np_arr1, np_arr2):
    """Construct polygon from the intersection part of two contours
    Parameters
    ----------    
    np_arr1 : np.array
        Contour with numpy array format
    np_arr2 : np.array
        Contour with numpy array format
    
    Returns
    -------
    inter_poly : Polygon
        The intersection polygon of two given polygons

    """
    inter_flag = contour_intersects(np_arr1, np_arr2)
    if inter_flag == False:
        return None
    # Construct polygon from numpy array
    poly1 = construct_poly_using_np_arr(np_arr1)
    poly2 = construct_poly_using_np_arr(np_arr2)
    # Get convex_hull of polygon
    poly1 = poly1.convex_hull  # Get convex hull of poly
    poly2 = poly2.convex_hull

    inter_poly = poly1.intersection(poly2)

    return inter_poly


def contour_contains(np_arr1, np_arr2):
    """Determine if contour of np_arr1 contains contour of np_arr2 or not
    Parameters
    ----------    
    np_arr1 : np.array
        Contour with numpy array format
    np_arr2 : np.array
        Contour with numpy array format
    
    Returns
    -------
    in_status : bool
        Check if polygon 1 contains polygon 2 or not
        
    """
    # Construct polygon from numpy array
    poly1 = construct_poly_using_np_arr(np_arr1)
    poly2 = construct_poly_using_np_arr(np_arr2)
    # Get convex_hull of polygon
    poly1 = poly1.convex_hull  # Get convex hull of poly
    poly2 = poly2.convex_hull

    in_status = poly1.contains(poly2)

    return in_status



# -*- coding: utf-8 -*-

import os, sys


__all__ = ["get_poly_area", "get_poly_bounds", "get_poly_wh"]

def get_poly_area(poly):
    """Calcualte the number of pixels Polygon covered
    Parameters
    ----------
    poly : Polygon
        Contour with shapely polygon format

    Returns
    -------
    area_val : int
        Number of pixels inside the contour

    """   

    area_val = poly.area

    return area_val


def get_poly_bounds(poly):
    """Find the bounds of the Polygon 
    Parameters
    ----------
    poly : Polygon
        Contour with shapely polygon format

    Returns
    -------
    min_x : int
        Minimum x coordinate of polygon
    min_y : int
        Minimum y coordinate of polygon    
    max_x : int
        Maximum x coordinate of polygon    
    max_y : int
        Maximum y coordinate of polygon        

    """   

    min_x, min_y, max_x, max_y = poly.bounds

    return min_x, min_y, max_x, max_y


def get_poly_wh(poly):
    """Find width and height of the Polygon 
    Parameters
    ----------
    poly : Polygon
        Contour with shapely polygon format

    Returns
    -------
    poly_w : int
        Width of the polygon
    poly_h : int
        Height of the polygon   

    """    
    min_x, min_y, max_x, max_y = get_poly_bounds(poly)
    poly_w, poly_h = max_x - min_x, max_y - min_y

    return poly_w, poly_h



# -*- coding: utf-8 -*-

import os, sys
import cv2
import numpy as np

__all__ = ["get_cnt_area", "get_cnt_aspect_ratio", "get_cnt_solidity"]


def get_cnt_area(cnt):
    """Calcualte the number of pixels contour covered
     Parameters
    ----------
    cnt : np.array
        Contour with opencv format

    Returns
    -------
    area_val : int
        Number of pixels inside the contour

    """
    area_val = cv2.contourArea(cnt.astype(np.float32))

    return area_val


def get_cnt_aspect_ratio(cnt):
    """Calcualte the aspect ratio of contour
     Parameters
    ----------
    cnt : np.array
        Contour with opencv format

    Returns
    -------
    aspect_ratio : float
        Value of contour aspect ratio  
    """    
    x, y, w, h = cv2.boundingRect(cnt.astype(np.float32))
    aspect_ratio = w * 1.0 / h

    return aspect_ratio


def get_cnt_solidity(cnt):
    """Calcualte the solidity of contour
     Parameters
    ----------
    cnt : np.array
        Contour with opencv format

    Returns
    -------
    solidity_val : float
        Solidity value of contour 
    """  

    cnt = cnt.astype(np.float32)
    area = cv2.contourArea(cnt)
    hull = cv2.convexHull(cnt)
    hull_area = cv2.contourArea(hull)

    solidity_val = area * 1.0 / hull_area

    return solidity_val



# -*- coding: utf-8 -*-

import os, sys

from pycontour import construct_poly_using_point_list
from pycontour.polygon import get_poly_area, get_poly_bounds, get_poly_wh

def test_property(poly):
    area = get_poly_area(poly)
    bounds = get_poly_bounds(poly)
    poly_w, poly_h = get_poly_wh(poly)
    print("Poly area is: {}".format(area))
    print("Poly bounds is: {}".format(bounds))
    print("Poly width is: {}, height is: {}".format(poly_w, poly_h))

if __name__ == "__main__":
    point_list1 = [(0, 1), (1, 2), (2, 1), (1, 0)]
    poly1 = construct_poly_using_point_list(point_list1)
    test_property(poly1)

    point_list2 = [(0, 0), (1, 2), (2, 0), (1, 0), (1, -1)]
    poly2 = construct_poly_using_point_list(point_list2)
    test_property(poly2)



# -*- coding: utf-8 -*-

import os, sys
import numpy as np

__all__ = ["cv_cnt_to_np_arr", "np_arr_to_cv_cnt"]

def cv_cnt_to_np_arr(cv_cnt):
    """Convert cv2 contour to normal numpy 2d array [0]-w, [1]-h
    Parameters
    ----------
    cv_cnt : np.array
        Contour with opencv format

    Returns
    -------
    cnt_arr : np.array
        Numpy array of point set     
    """
    cnt_arr = cv_cnt.squeeze().transpose()

    return cnt_arr

def np_arr_to_cv_cnt(np_arr):
    """Convert 2d coordinates to cv2 contour
    Parameters
    ----------
    cnt_arr : np.array
        Numpy array of point set    
    Returns
    -------
    cv_cnt : np.array
        Contour with opencv format
                    
    """
    arr_t = np_arr.transpose()
    cv_cnt = np.expand_dims(arr_t, axis=1)

    return cv_cnt



# -*- coding: utf-8 -*-

import os, sys
import numpy as np
import scipy.misc as misc
from scipy.ndimage import binary_fill_holes
from skimage import img_as_ubyte
from skimage.morphology import remove_small_objects
from skimage.morphology import disk, binary_closing
import cv2
import matplotlib.pyplot as plt
from pycontour import DATA_DIR


def extract_cnt_using_cv2(img_path):
    img = misc.imread(img_path)
    binary = img > 156
    no_small = remove_small_objects(binary, min_size=5000, connectivity=8)
    fill = binary_fill_holes(no_small)
    selem = disk(10)
    mask = binary_closing(fill, selem)
    ubyte_mask = img_as_ubyte(mask)
    _, cnts, _ = cv2.findContours(ubyte_mask, mode=cv2.RETR_EXTERNAL, method=cv2.CHAIN_APPROX_NONE)

    return cnts

if __name__ == "__main__":
    img_path = os.path.join(DATA_DIR, "images", "brain.jpg")
    cnts = extract_cnt_using_cv2(img_path)



# -*- coding: utf-8 -*-

import os, sys
import numpy as np
from pycontour import np_arr_to_point_list
from pycontour import construct_poly_using_np_arr
from pycontour import construct_poly_using_point_list
from pycontour import construct_poly_using_bbox
from pycontour import poly_to_np_arr


if __name__ == "__main__":
    np_arr = np.array([[1, 2, 4, 5, 3], [1, 3, 4, 2, 0]])
    poly1 = construct_poly_using_np_arr(np_arr)
    print("Bounds of poly1 is: ", poly1.exterior.bounds)
    point_list = np_arr_to_point_list(np_arr)
    poly2 = construct_poly_using_point_list(point_list)
    print("Bounds of poly2 is: ", poly2.exterior.bounds)
    min_w, max_w = np.min(np_arr[0]), np.max(np_arr[0])
    min_h, max_h = np.min(np_arr[1]), np.max(np_arr[1])
    poly3 = construct_poly_using_bbox(min_w, min_h, max_w, max_h)
    print("Bounds of poly3 is: ", poly3.exterior.bounds)
    poly1_arr = poly_to_np_arr(poly1)
    print("Numpy coordinates of poly1 is:")
    print(poly1_arr)



# -*- coding: utf-8 -*-

import os, sys
import numpy as np
from shapely.geometry import Polygon, box

from pycontour import np_arr_to_point_list
from pycontour import point_list_to_np_arr

__all__ = ["construct_poly_using_np_arr", "construct_poly_using_point_list",
    "construct_poly_using_bbox", "poly_to_np_arr"]


def construct_poly_using_np_arr(np_arr):
    """Using numpy array to construct polygon
    Parameters
    ----------
    np_arr : np.array
        Numpy array of point set    

    Returns
    -------
    poly : Polygon
        Contour with shapely polygon format    

    """
    point_list = np_arr_to_point_list(np_arr)
    poly = Polygon(point_list)

    return poly


def construct_poly_using_point_list(point_list):
    """Using point list to construct polygon
    Parameters
    ----------
    point_list : list
        List of point set    

    Returns
    -------
    poly : Polygon
        Contour with shapely polygon format

    """
    poly = Polygon(point_list)

    return poly


def construct_poly_using_bbox(min_w, min_h, max_w, max_h):
    """Using bounding box to construct polygon
    Parameters
    ----------
    min_w : int
        Minimum x coordinate of polygon
    min_h : int 
        Minimum y coordinate of polygon        
    max_w : int
        Maximum x coordinate of polygon
    max_h : int 
        Maximum y coordinate of polygon  

    Returns
    -------
    poly : Polygon
        Contour with shapely polygon format  

    """
    poly = box(min_w, min_h, max_w, max_h)

    return poly


def poly_to_np_arr(poly):
    """Convert shapely Polygon to normal numpy 2d array [0]-w, [1]-h
    Parameters
    ----------
    poly : Polygon
        Contour with shapely polygon format  

    Returns
    -------
    cnt_arr : np.array
        Numpy array of point set
    """
    x_coors, y_coors = poly.exterior.coords.xy
    x_coors = x_coors[:-1].tolist()
    y_coors = y_coors[:-1].tolist()
    point_list = [(x, y) for x, y in zip(x_coors, y_coors)]
    cnt_arr = point_list_to_np_arr(point_list)

    return cnt_arr



# -*- coding: utf-8 -*-

import os, sys
import numpy as np

__all__ = ["swap_wh", "np_arr_to_point_list", "point_list_to_np_arr"]


def swap_wh(np_arr):
    """Swap row of width and row of height
     Parameters
    ----------
    np_arr : np.array
        Numpy array with width and height

    Returns
    -------
    np_arr : np.array
        Numpy array with width and height position change 
  
    """

    np_arr[[0,1]] = np_arr[[1,0]]

    return np_arr

def np_arr_to_point_list(np_arr):
    """Convert 2d numpy array to list of points
     Parameters
    ----------
    np_arr : np.array
        Numpy array of point set

    Returns
    -------
    point_list : list
        List of point set        
    """

    point_list = []
    num_point = np_arr.shape[1]
    for ind in range(num_point):
        point_list.append((np_arr[0][ind], np_arr[1][ind]))

    return point_list

def point_list_to_np_arr(point_list):
    """Convert list of point coordinates to 2d array
     Parameters
    ----------
    point_list : list
        List of point set            

    Returns
    -------
    np_arr : np.array
        Numpy array of point set    
    
    """
    num_point = len(point_list)
    np_arr = np.zeros((2, num_point))
    for ind in range(num_point):
        np_arr[0][ind] = point_list[ind][0]
        np_arr[1][ind] = point_list[ind][1]

    return np_arr



# -*- coding: utf-8 -*-

import os, sys

from pycontour import point_list_to_np_arr
from pycontour.polygon import contour_intersects
from pycontour.polygon import construct_intersection_polygon


if __name__ == "__main__":
    point_list1 = [(1, 4), (3, 4), (3, 1), (1, 1)]
    point_list2 = [(2, 3), (4, 3), (4, 0), (2, 0)]
    point_list3 = [(3, 1), (5, 1), (5, -2), (3, -2)]
    # Convert point list to numpy array
    np_arr1 = point_list_to_np_arr(point_list1)
    np_arr2 = point_list_to_np_arr(point_list2)
    np_arr3 = point_list_to_np_arr(point_list3)

    # inter12 = contour_intersects(np_arr1, np_arr2)
    # print("Point list1 and point list2 intersected: {}".format(inter12))
    # inter13 = contour_intersects(np_arr1, np_arr3)
    # print("Point list1 and point list3 intersected: {}".format(inter13))
    # inter23 = contour_intersects(np_arr2, np_arr3)
    # print("Point list2 and point list3 intersected: {}".format(inter23))
    intersection12 = construct_intersection_polygon(np_arr1, np_arr2)
    print("Area of intersection 1 and 2 is: {}".format(intersection12.area))
    intersection13 = construct_intersection_polygon(np_arr1, np_arr3)
    print("Area of intersection 1 and 3 is: {}".format(intersection13.area))
    intersection23 = construct_intersection_polygon(np_arr2, np_arr3)
    print("Area of intersection 2 and 3 is: {}".format(intersection23.area))



# -*- coding: utf-8 -*-

import os, sys
import scipy.misc as misc
import cv2
import matplotlib.pyplot as plt

from pycontour import DATA_DIR
from pycontour import cv_cnt_to_np_arr, np_arr_to_cv_cnt

from .extract_cv2_cnts import extract_cnt_using_cv2

def test_cv_np_transfom(tmp_cnt):
    # convert cv2 contour to numpy array
    np_arr = cv_cnt_to_np_arr(tmp_cnt)
    # convert numpy arrary to cv2 contour
    cv_cnt = np_arr_to_cv_cnt(np_arr)

    return cv_cnt


if __name__ == "__main__":
    img_path = os.path.join(DATA_DIR, "images", "brain.jpg")
    img = misc.imread(img_path)
    cnts = extract_cnt_using_cv2(img_path)
    test_cnt = cnts[1]
    cv_cnt = test_cv_np_transfom(test_cnt)
    draw_img = cv2.drawContours(img, [cv_cnt], 0, 255, 3)



# -*- coding: utf-8 -*-

import os, sys

from pycontour import point_list_to_np_arr
from pycontour import swap_wh
from pycontour import np_arr_to_point_list


if __name__ == "__main__":
    import pdb; pdb.set_trace()
    point_list = [(0, 0), (2, 0), (2, 2), (1, 2)]
    np_arr = point_list_to_np_arr(point_list)
    swap_arr = swap_wh(np_arr)
    swap_points = np_arr_to_point_list(swap_arr)



# -*- coding: utf-8 -*-

import os, sys

__all__ = ["PKG_DIR", "DATA_DIR", "__version__"]

__version__ = '1.0.0'
PKG_DIR = os.path.abspath(os.path.dirname(__file__))
DATA_DIR = os.path.join(os.path.dirname(PKG_DIR), 'data')


from .coor_transform import *
from .cv2_transform import *
from .poly_transform import *
